""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.2.0
Author  : Stefano Covino
Date    : 27/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input date are lists of x and weights

History : (28/10/2010) First version.
        : (03/08/2011) Correction in weighting.
        : (27/08/2012) Better managment of invalid floating point operations and of weights arrays.

"""


import math

import numpy


def WeightedMeanFrame(x,wx=None):
    oldsettings = numpy.seterr(all='ignore')
    nx = []
    nwx = []
    # generate numpy arrays
    for i in range(len(x)):
        nx.append(numpy.array(x[i]))
        if wx != None:
            nwx.append(numpy.array(wx[i]))
        else:
            nwx.append(numpy.ones(nx[0].shape))
    #
    if len(nx) == 1:
        return nx[0], numpy.zeros(nx[0].shape), numpy.zeros(nx[0].shape), numpy.ones(nx[0].shape)
    #    
    numx = numpy.zeros(nx[0].shape)
    denw = numpy.zeros(nx[0].shape)
    #
    for i in range(len(nx)):
        numx = numx + nx[i] * nwx[i]
        denw = denw + nwx[i]
    # mean
    wa = numpy.divide(numx,denw)
    # var
    numx = numpy.zeros(nx[0].shape)
    #
    for i in range(len(nx)):
        numx = numx + (nx[i]-wa)**2
    if len(nx) > 1:
        was = numpy.divide(numx,len(nx)-1)
    else:
        was = numpy.zeros(nx[0].shape)
    # std 
    ws = numpy.sqrt(was)
    # 
    we = numpy.divide(ws,math.sqrt(len(nx)))
    # exposure map
    wexp = numpy.divide(denw,numpy.max(denw))
    #
    numpy.seterr(**oldsettings)
    return numpy.nan_to_num(wa),numpy.nan_to_num(ws),numpy.nan_to_num(we),numpy.nan_to_num(wexp)




""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 19/05/2021
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : emcee MCMC
        : parameter:
           nwalkers - number of walkers
           starts - starting positions of the chain
           sigmas - sigmas for the starting Gaussian ball (e.g. 0.1*starts)
           nburnt - ingterations to reach stability
           niters - total iterations
           lnprob - log of probability to sample
           lnargs - [parameters for lnprob]


        : Example of code
          def lnlike(theta, an, So, eSo):
          #
           smod = model(theta, an)
           return -0.5 * np.sum(np.log(2 * np.pi * eSo ** 2) + (So - smod) ** 2 / eSo ** 2)

          def lnprior(theta):
           prior = 0.
           #
           P = np.exp(theta[0])
           Pos = np.degrees(theta[1])
           #
           if P > 1:
            return -np.inf
           #
           if Pos < -180 or Pos > 180:
            return -np.inf
           #
           return prior

           def lnprob(theta, an, So, eSo):
            theta[1] = phaseang(theta[1])
            lp = lnprior(theta)
            if not np.isfinite(lp):
             return -np.inf
            return lp + lnlike(theta, an, So, eSo)

            def model(theta, an):
             anr = np.radians(an)
             #
             P = np.exp(theta[0])
             Pos = theta[1]
             #
             return P*np.cos(2*(Pos-anr))


History : (05/08/2020) First version.
		: (21/01/2021) Improved running.
"""

import emcee
import numpy as np


def emcee_run (nwalkers,starts,sigmas,niters,lnprob,lpargs,fjump=4,nthin=1):
    
    ndim = len(starts)

    pos = emcee.utils.sample_ball(starts,sigmas,nwalkers)

    sampler = emcee.EnsembleSampler(nwalkers, ndim, lnprob, args=lpargs)

    npos = sampler.run_mcmc(pos, niters)
    
    try:
        atime = np.mean(sampler.get_autocorr_time())
    except:
        atime = 0
        
    if np.isnan(atime):
        atime = 0
    
    samples = sampler.get_chain(discard=int(fjump*atime), thin=nthin, flat=True)

    meanaccfract = np.mean(sampler.acceptance_fraction)

    return {"sampler": sampler, "meanacceptancefraction": meanaccfract, "samples": samples, "autocorrtime":atime, "lensamples":len(samples)}




""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 17/01/2022
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : RebinData(array,width,sigmaclip=None)
          The function returns an array formed by sigma clipped
          mean in sub-arrays of given width. For the regular mean 
          put sigmaclip to None.

History : (22/12/2014) First version.
        : (17/01/2022) standard deviation in output.

"""

import astropy.stats.funcs as asf
import numpy


def RebinData(array,width,sigmaclip=None):
    rebarr = array[:(array.size // width) * width].reshape(-1, width)
    if sigmaclip != None:
        return asf.sigma_clip(rebarr,sig=sigmaclip,axis=1).mean(axis=1), asf.sigma_clip(rebarr,sig=sigmaclip,axis=1).std(axis=1)
    else:
        return rebarr.mean(axis=1), rebarr.std(axis=1)
 
    


""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 29/04/2022
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : BIC (loglikelihood,npars,ndata)
            This function reports the BIC knowking the best-fit Log Likelihoood,
            the number of data-points and the number of fit parameters.

Remarks :

History : (17/01/2022) First version.
        : (29/04/2022) Minor bug corrected.

"""


import numpy as np


def BIC(LLike,npars,ndata):
    return -2*LLike+npars*np.log(ndata)


    
    


""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 04/03/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : StDev(table,mean=None)
            table is the dataset and mean is the point to be used for mean.
          The function return the standard deviation
        

Remarks : input is an array

History : (30/04/2011) First version.

"""


import numpy


def StDev(table,mean=None):
    if mean == None:
        mn = numpy.mean(table)
    else:
        mn = mean 
    StDev = numpy.sqrt(((table-mn)**2).sum()/len(table))
    return StDev


    
    


""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 04/03/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : MAD(table)
            table is the dataset.
          The function return the Median Absolute Deviation
        

Remarks : input is a list of (x,sigmax)

History : (30/04/2011) First version.

"""


import numpy


def MAD(table):
    median = numpy.median(table)
    mad = numpy.median(numpy.abs(table-median))
    return mad


    
    


""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 22/06/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (22/06/2010) First version.

"""


import math, random


def AsymmetricGaussianDistrib(m,sl,sr):
    discrfactor = float(math.fabs(sl))/(float(math.fabs(sl))+float(math.fabs(sr)))
    trial1 = random.uniform(0.0,1.0)
    if trial1 < discrfactor:
        # left
        extrsigma = math.fabs(random.gauss(0,math.fabs(sl)))
        finpar = m-extrsigma
    else:
        # right
        extrsigma = math.fabs(random.gauss(0,math.fabs(sr)))
        finpar = m+extrsigma
    return finpar



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/02/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : FourierPeriodogram

History : (03/02/2020) First version.

"""

import numpy as np


def ACF(x):
    n = len(x)
    variance = x.var()
    x = x-x.mean()
    r = np.correlate(x, x, mode = 'full')[-n:]
    #assert N.allclose(r, N.array([(x[:n-k]*x[-(n-k):]).sum() for k in range(n)]))
    result = r/(variance*(np.arange(n, 0, -1)))
    return result



""" Utility functions and classes for SRP

Context : SRP
Module  : Statistics.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 03/08/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input data are lists of x and weights

History : (29/10/2010) First version.
        : (03/08/2011) Correction to weighting.

"""



import numpy

from .WeightedMeanFrame import WeightedMeanFrame




def CondSum (cumnum,cumden,x,wx,cond,it):
    r1 = (numpy.add(cumnum,numpy.multiply(x,numpy.power(wx,-2))),numpy.add(cumden,numpy.power(wx,-2)),it+1)
    r2 = (cumnum,cumden,it)
    return numpy.where (cond, r1, r2) 



def CondVar (cumnum,x,wa,cond,it):
    r1 = (numpy.add(cumnum,numpy.power(numpy.subtract(x,wa),2)),it+1)
    r2 = (cumnum,it)
    return numpy.where (cond, r1, r2) 



def AverSigmaClippFrame (x, wx=None, downsig=None, upsig=None, alla=False):
    nx = []
    nwx = []
    # generate numpy arrays
    for i in range(len(x)):
        nx.append(numpy.array(x[i]))
        if wx != None:
            nwx.append(numpy.array(wx[i]))
        else:
            nwx.append(numpy.ones(nx[0].shape))
    #
    res = WeightedMeanFrame(nx,nwx)
    wa = res[0]
    ws = res[1]
    we = res[2]
    wexp = res[3]
    #
    if (downsig == None and upsig == None) or (len(nx) == 1):
        # no condition, nothing to do
        if alla:
            return wa,wexp,ws,we
        else:
            return wa,wexp
    #
    numx = numpy.zeros(nx[0].shape)
    denw = numpy.zeros(nx[0].shape)
    nit  = numpy.zeros(nx[0].shape) 
    #
    for i in range(len(nx)):
        if downsig != None and upsig == None:
            # high-pass filter
            condiz = (nx[i] >= numpy.subtract(wa,numpy.multiply(ws,downsig)))
        elif downsig == None and upsig != None:
            # low-pass filter
            condiz = (nx[i] <= numpy.add(wa,numpy.multiply(ws,upsig)))
        else:
            # sigma-clipping
            condiz = (numpy.logical_and((nx[i] >= numpy.subtract(wa,numpy.multiply(ws,downsig))),(nx[i] <= numpy.add(wa,numpy.multiply(ws,upsig)))))
        #
        resi = CondSum (numx,denw,nx[i],nwx[i],condiz,nit)
        #
        numx = resi[0]
        denw = resi[1]
        nit = resi[2]
    #
    # mean
    waclp = numpy.divide(numx,denw)
    # exposure map
    wexpclp = numpy.divide(denw,numpy.max(denw))
    #
    # Go on computing only if explicitly required
    if not alla:
        return waclp,wexpclp
    # var
    numx = numpy.zeros(nx[0].shape)
    nit  = numpy.zeros(nx[0].shape) 
    #
    for i in range(len(nx)):
        if downsig != None and upsig == None:
            # high-pass filter
            condiz = (nx[i] >= numpy.subtract(wa,numpy.multiply(ws,downsig)))
        elif downsig == None and upsig != None:
            # low-pass filter
            condiz = (nx[i] <= numpy.add(wa,numpy.multiply(ws,upsig)))
        else:
            # sigma-clipping
            condiz = (numpy.logical_and((nx[i] >= numpy.subtract(wa,numpy.multiply(ws,downsig))),(nx[i] <= numpy.add(wa,numpy.multiply(ws,upsig)))))
        #
        resi = CondVar (numx,nx[i],waclp,condiz,nit)
        #
        numx = resi[0]
        nit = resi[1] 
    #
    wasclp = numpy.where(nit>1, numpy.divide(numx,nit-1), numpy.zeros(nx[0].shape))
    # std and sterr
    wsclp = numpy.sqrt(wasclp)
    weclp = numpy.where(nit>1, numpy.divide(wsclp,numpy.sqrt(nit)), numpy.zeros(nx[0].shape))
    #
    return waclp,wexpclp,wsclp,weclp





""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 04/09/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (04/00/2010) First version.

"""


import math
import numpy
import scipy.optimize as so

from SRP.SRPMath.CartesianCoordAmpRotoTranslation import CartesianCoordAmpRotoTranslation

# vars
#   0 x,y
#   1 rx,ry
# pars
#   0   xshift
#   1   yshift
#   2   rotangle
#   3   amplfactor

def SquareDistSum (parms, oldvars, newvars):
    tot = 0.0
    for i,l in zip(oldvars,newvars):
        newc = CartesianCoordAmpRotoTranslation((i[0],i[1]),(parms[0],parms[1]),parms[2],math.fabs(parms[3]))
#        print i, newc, l
        tot = tot + math.sqrt((l[0]-newc[0])**2 + (l[1]-newc[1])**2)
    return tot


def CoordDistanceMinimization (refcoord, oldcoord, guessparset):
    rc = numpy.array(refcoord)
    oc = numpy.array(oldcoord)
    xopt = so.fmin(SquareDistSum, guessparset, args=(oc,rc), disp=0)
    return xopt, SquareDistSum(xopt,oc,rc)/len(oc)
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.1.1
Author  : Stefano Covino
Date    : 13/10/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input is a list of (x,sigmax)

History : (21/05/2010) First version.
        : (20/08/2010) Input list with no weights case.
        : (13/10/2010) Single object case.

"""

from . import WeightedMean as WM
import copy


def AverIterSigmaClipp (inplst, siglev=2.0):
    worklist = copy.deepcopy(inplst)
    wlst = []
    for i in worklist:
        wlst.append(True)
    ocard = len(wlst)   
    #
    if len(worklist) > 0:
        try:
            ndt = len(worklist[0])
        except TypeError:
            ndt = 1
        if ndt < 2:
            for i in range(len(worklist)):
                worklist[i] = (worklist[i],1.0)
    #
    while True:
        if len(worklist) < 1:
            return None,None,None,None
        elif len(worklist) == 1:
            return worklist[0][0],worklist[0][1],worklist[0][1],1.0,1
        anlst = []
        for i in range(len(wlst)):
            if wlst[i] == True:
                anlst.append(worklist[i])
                #print worklist[i]
        res = WM.WeightedMean(anlst)
        wa = res[0]
        ws = res[1]
        we = res[2]
        ch = res[3]
        #print res
        wmin, wmax = wa - siglev*ws, wa + siglev*ws
        for i in range(len(wlst)):
            if not wmin <= worklist[i][0] <= wmax:
                wlst[i] = False
            #print wmin,wmax,worklist[i][0],wa,ws
        card = 0
        for i in wlst:
            if i == True:
                card = card + 1
        if card != ocard:
                ocard = card
        else:
            return wa,ws,we,ch,len(anlst)




""" Utility functions and classes for SRP

Context : SRP
Module  : FT
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/01/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : From https://gist.github.com/astroboxio/4063132

History : (29/01/2020) First version.

"""


import numpy as np

try:
    # use scipy if available: it's faster
    from scipy.fftpack import fft, ifft, fftshift, ifftshift
except:
    from numpy.fft import fft, ifft, fftshift, ifftshift


def FT_continuous(t, h, axis=-1, method=1):
    """Approximate a continuous 1D Fourier Transform with sampled data.

    This function uses the Fast Fourier Transform to approximate
    the continuous fourier transform of a sampled function, using
    the convention

    .. math::

       H(f) = \int h(t) exp(-2 \pi i f t) dt

    It returns f and H, which approximate H(f).

    Parameters
    ----------
    t : array_like
        regularly sampled array of times
        t is assumed to be regularly spaced, i.e.
        t = t0 + Dt * np.arange(N)
    h : array_like
        real or complex signal at each time
    axis : int
        axis along which to perform fourier transform.
        This axis must be the same length as t.

    Returns
    -------
    f : ndarray
        frequencies of result.  Units are the same as 1/t
    H : ndarray
        Fourier coefficients at each frequency.
    """
    assert t.ndim == 1
    assert h.shape[axis] == t.shape[0]
    N = len(t)
    if N % 2 != 0:
        raise ValueError("number of samples must be even")

    Dt = t[1] - t[0]
    Df = 1. / (N * Dt)
    t0 = t[N // 2]

    f = Df * (np.arange(N) - N / 2)

    shape = np.ones(h.ndim, dtype=int)
    shape[axis] = N

    phase = np.ones(N)
    phase[1::2] = -1
    phase = phase.reshape(shape)

    if method == 1:
        H = Dt * fft(h * phase, axis=axis)
    else:
        H = Dt * fftshift(fft(h, axis=axis), axes=axis)

    H *= phase
    H *= np.exp(-2j * np.pi * t0 * f.reshape(shape))
    H *= np.exp(-1j * np.pi * N / 2)

    return f, H


def IFT_continuous(f, H, axis=-1, method=1):
    """Approximate a continuous 1D Inverse Fourier Transform with sampled data.

    This function uses the Fast Fourier Transform to approximate
    the continuous fourier transform of a sampled function, using
    the convention

    .. math::

       H(f) = integral[ h(t) exp(-2 pi i f t) dt]

       h(t) = integral[ H(f) exp(2 pi i f t) dt]

    It returns t and h, which approximate h(t).

    Parameters
    ----------
    f : array_like
        regularly sampled array of times
        t is assumed to be regularly spaced, i.e.
        f = f0 + Df * np.arange(N)
    H : array_like
        real or complex signal at each time
    axis : int
        axis along which to perform fourier transform.
        This axis must be the same length as t.

    Returns
    -------
    f : ndarray
        frequencies of result.  Units are the same as 1/t
    H : ndarray
        Fourier coefficients at each frequency.
    """
    assert f.ndim == 1
    assert H.shape[axis] == f.shape[0]
    N = len(f)
    if N % 2 != 0:
        raise ValueError("number of samples must be even")

    f0 = f[0]
    Df = f[1] - f[0]

    t0 = -0.5 / Df
    Dt = 1. / (N * Df)
    t = t0 + Dt * np.arange(N)

    shape = np.ones(H.ndim, dtype=int)
    shape[axis] = N

    t_calc = t.reshape(shape)
    f_calc = f.reshape(shape)

    H_prime = H * np.exp(2j * np.pi * t0 * f_calc)
    h_prime = ifft(H_prime, axis=axis)
    h = N * Df * np.exp(2j * np.pi * f0 * (t_calc - t0)) * h_prime

    return t, h


def PSD_continuous(t, h, axis=-1, method=1):
    """Approximate a continuous 1D Power Spectral Density of sampled data.

    This function uses the Fast Fourier Transform to approximate
    the continuous fourier transform of a sampled function, using
    the convention

    .. math::

        H(f) = \int h(t) \exp(-2 \pi i f t) dt

    It returns f and PSD, which approximate PSD(f) where

    .. math::

        PSD(f) = |H(f)|^2 + |H(-f)|^2

    Parameters
    ----------
    t : array_like
        regularly sampled array of times
        t is assumed to be regularly spaced, i.e.
        t = t0 + Dt * np.arange(N)
    h : array_like
        real or complex signal at each time
    axis : int
        axis along which to perform fourier transform.
        This axis must be the same length as t.

    Returns
    -------
    f : ndarray
        frequencies of result.  Units are the same as 1/t
    PSD : ndarray
        Fourier coefficients at each frequency.
    """
    assert t.ndim == 1
    assert h.shape[axis] == t.shape[0]
    N = len(t)
    if N % 2 != 0:
        raise ValueError("number of samples must be even")

    ax = axis % h.ndim

    if method == 1:
        # use FT_continuous
        f, Hf = FT_continuous(t, h, axis)
        Hf = np.rollaxis(Hf, ax)
        f = -f[N // 2::-1]
        PSD = abs(Hf[N // 2::-1]) ** 2
        PSD[:-1] += abs(Hf[N // 2:]) ** 2
        PSD = np.rollaxis(PSD, 0, ax + 1)
    else:
        # A faster way to do it is with fftshift
        # take advantage of the fact that phases go away
        Dt = t[1] - t[0]
        Df = 1. / (N * Dt)
        f = Df * np.arange(N / 2 + 1)
        Hf = fft(h, axis=axis)
        Hf = np.rollaxis(Hf, ax)
        PSD = abs(Hf[:N // 2 + 1]) ** 2
        PSD[-1] = 0
        PSD[1:] += abs(Hf[N // 2:][::-1]) ** 2
        PSD[0] *= 2
        PSD = Dt ** 2 * np.rollaxis(PSD, 0, ax + 1)

    return f, PSD


def sinegauss(t, t0, f0, Q):
    """Sine-gaussian wavelet"""
    a = (f0 * 1. / Q) ** 2
    return (np.exp(-a * (t - t0) ** 2)
            * np.exp(2j * np.pi * f0 * (t - t0)))


def sinegauss_FT(f, t0, f0, Q):
    """Fourier transform of the sine-gaussian wavelet.

    This uses the convention

    .. math::

       H(f) = integral[ h(t) exp(-2pi i f t) dt]
    """
    a = (f0 * 1. / Q) ** 2
    return (np.sqrt(np.pi / a)
            * np.exp(-2j * np.pi * f * t0)
            * np.exp(-np.pi ** 2 * (f - f0) ** 2 / a))


def sinegauss_PSD(f, t0, f0, Q):
    """Compute the PSD of the sine-gaussian function at frequency f

    .. math::

       PSD(f) = |H(f)|^2 + |H(-f)|^2
    """
    a = (f0 * 1. / Q) ** 2
    Pf = np.pi / a * np.exp(-2 * np.pi ** 2 * (f - f0) ** 2 / a)
    Pmf = np.pi / a * np.exp(-2 * np.pi ** 2 * (-f - f0) ** 2 / a)
    return Pf + Pmf


def wavelet_PSD(t, h, f0, Q=1.0):
    """Compute the wavelet PSD as a function of f0 and t

    Parameters
    ----------
    t : array_like
        array of times, length N
    h : array_like
        array of observed values, length N
    f0 : array_like
        array of candidate frequencies, length Nf
    Q : float
        Q-parameter for wavelet

    Returns
    -------
    PSD : ndarray
        The 2-dimensional PSD, of shape (Nf, N), corresponding with
        frequencies f0 and times t.
    """
    t, h, f0 = map(np.asarray, (t, h, f0))
    if (t.ndim != 1) or (t.shape != h.shape):
        raise ValueError('t and h must be one dimensional and the same shape')

    if f0.ndim != 1:
        raise ValueError('f0 must be one dimensional')

    Q = Q + np.zeros_like(f0)

    f, H = FT_continuous(t, h)
    W = np.conj(sinegauss_FT(f, 0, f0[:, None], Q[:, None]))
    _, HW = IFT_continuous(f, H * W)

    return abs(HW) ** 2



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.2
Author  : Stefano Covino
Date    : 18/11/2021
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input is a list of (x,sigmax)

History : (19/02/2012) First version.
        : (27/03/2013) Homogenous output in case of single output.
        : (18/11/2021) Minor correction.
"""


import numpy


def GenGaussSet (vl, evl, ntrial=100):
    if evl != 0.0:
        return numpy.random.normal(vl,evl,ntrial)
    else:
        return numpy.array([vl])



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats
Version : 1.2.0
Author  : Stefano Covino
Date    : 19/04/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input is a list of (x,sigmax)

History : (21/05/2010) First version.
        : (20/04/2012) Case with no data considered.
        : (19/04/2014) Choice between true standard deviation o not.
"""


import math


def WeightedMean(lst,truedevst=False):
    if len(lst) < 1:
        return 0,0,0,0
    elif len(lst) == 1:
        return lst[0][0], lst[0][1], lst[0][1], 1.
    #
    num = 0.0
    den = 0.0
    for i in lst:
        num = num + i[0]*i[1]**-2
        den = den + i[1]**-2
    wa = num/den
    sum = 0.0
    for i in lst:
        sum = sum + (wa - i[0])**2
    if truedevst:
        we = math.sqrt(1./den)
        ws = we*math.sqrt(len(lst))
    else:
        ws = math.sqrt(sum/(len(lst)-1))
        we = ws/math.sqrt(len(lst))
    rchis = 0.0
    for i in lst:
        rchis = rchis + ((i[0]-wa)/i[1])**2
    rchis = rchis/(len(lst)-1)
    return wa,ws,we,rchis



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats
Version : 1.0.0
Author  : Stefano Covino
Date    : 13/08/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino
Remark  : This is the so-called Posterior predictive p-value (Gelman, A.,
            Carlin, J. B., Stern, H. S., et al. 2013, Bayesian Data Analysis
            (Boca Raton, FL: CRC Press). Commented in Lucy (2018, A&A 614,
            25) and Lucy (2018, 

Usage   : Bayespvalue(model,modelsim data, edata)

Example:
        # Derive a set of models based on the posterior distribution
        S =  model([samples[:,[0]],samples[:,[1]]],dt['angle'])
        scl = np.tile(dt['err_S'],samples[:,[0]].shape[0])
        Ss = S.reshape(-1) + SS.norm.rvs(scale=scl,size=S.size,loc=0)
        Ssim = Ss.reshape(S.shape)

        model: S
	modelsim: Ssim

History : (13/08/2020) First version.
"""

from SRPSTATS.chi2 import chi2
import numpy as np


def Bayespvalue (model, modelsim, data, edata):
    #
    a = chi2(model,modelsim,edata)
    b = chi2(model,data,edata)
    c = (a > b)
    return {"Bayes-p-value": np.count_nonzero(c)/len(c)}



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 03/06/2021
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : BayesFactor(e1,ee1,e2,ee2)
            e1,e2 are the computed evidence, ee1 and ee2 the errors.
          The function return the a dictorionary with e Bayes factor and the
          associated probabilities with error.
        
Remarks : 

History : (04/10/2019) First version.
        : (03/06/2021) Porting to SRPSTATS.
"""


import numpy as np
from SRPSTATS.ScoreatPercentile import ScoreatPercentile
from SRPSTATS.GenGaussSet import GenGaussSet

def BayesFactor (p1, ep1, p2, ep2):
    p1set = GenGaussSet(p1,ep1,10000)
    p2set = GenGaussSet(p2,ep2,10000)
    #p1l = np.exp(p1)
    #ep1l = p1l*ep1
    #p2l = np.exp(p2)
    #ep2l = p2l*ep2
    #
    lnbf = p2-p1
    elnbf = np.sqrt(ep1**2+ep2**2)
    #
    bf = np.exp(p2-p1)
    #ebf = bf*np.sqrt(ep1**2 + ep2**2)
    bfset = np.exp(p2set-p1set)
    ebf = ScoreatPercentile(bfset)[3]
    #
    #bfl = p2l/p1l
    #ebfl = bfl*np.sqrt((ep1l/p1l)**2+(ep2l/p2l)**2)
    #
    #pl = p2l/(p1l+p2l)
    #sl = p1l+p2l
    #esl = np.sqrt(ep1l**2+ep2l**2)
    #epl = pl*np.sqrt((ep2l/p2l)**2 + (esl/sl)**2)
    #
    p = 1./(1+np.exp(p1-p2))
    #ep = (np.exp(p1-p2)*p**2)*np.sqrt(ep1**2+ep2**2)
    pset = 1./(1+np.exp(p1set-p2set))
    ep = ScoreatPercentile(pset)[3]
    return {"BF": bf, "eBF": ebf, "p":p, "ep":ep, "lnBF":lnbf, "elnBF":elnbf}




""" Utility functions and classes for SRP

Context : SRP
Module  : Stats
Version : 1.0.0
Author  : Stefano Covino
Date    : 13/08/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : pvalue(model, data, edata, nfitpars)


History : (13/08/2020) First version.
"""

from SRPSTATS.chi2 import chi2
import scipy.stats as SS


def pvalue (model, data, edata, npars):
    #
    chiquadro = chi2(model,data,edata)
    chi2red = chiquadro/(len(data)-npars)
    #
    return {'chi2':chiquadro, 'chi2red':chi2red, "p-value":1-SS.chi2.cdf(float(chiquadro),len(data)-npars), "dof":len(data)-npars}
#


""" Utility functions and classes for SRP

Context : SRP
Module  : Stats
Version : 1.0.0
Author  : Stefano Covino
Date    : 31/07/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : chi2(m,o,eo)
            m are the model data
            o are the observations
            eo are the 1sigma errors for the observations

History : (31/07/2020) First version.
"""



def chi2 (m,o,eo):
    c2 = ((m-o)/eo)**2
    if len(c2.shape) == 2:
        return c2.sum(axis=1)
    else:
        return c2.sum()



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.1.1
Author  : Stefano Covino
Date    : 18/11/2021
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input is a list of (x,sigmax)

History : (19/02/2012) First version.
        : (27/03/2013) Output in case of single entry.
        : (29/07/2015) numpy porting.
        : (18/11/2021) Quantile correction.
"""


import numpy
from scipy.stats import scoreatpercentile


def ScoreatPercentile(vl, percdown=15.85, percc=50.0, percup=84.15):
    if len(vl) == 1:
        return list(vl)[0],list(vl)[0],list(vl)[0],0.0
    if numpy.std(list(vl)) != 0.0:
        vd = scoreatpercentile(list(vl),percdown)
        vc = scoreatpercentile(list(vl),percc)    
        vu = scoreatpercentile(list(vl),percup)
        return vd,vc,vu,(vu-vd)/2.0
    else:
        return list(vl)[0],list(vl)[0],list(vl)[0],0.0



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/11/2016
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : FourierPeriodogram

History : (02/11/2016) First version.

"""

import numpy as np


def FourierPeriodogram(t, y, minf=None, maxf=None):
    N = len(t)
    step = t[1] - t[0]
    frequency = np.fft.fftfreq(N, step)
    y_fft = np.fft.fft(y)
    if minf != None and maxf != None:
        positive = ((frequency > 0) & (frequency >= minf) & (frequency <= maxf))
    else:
        positive = (frequency > 0)
    return frequency[positive], (1./N) * abs(y_fft[positive]) ** 2




""" Utility functions and classes for SRP

Context : SRP
Module  : GP
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/01/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : From C. E. Rasmussen & C. K. I. Williams, Gaussian Processes
            for Machine Learning, the MIT Press, 2006

History : (29/01/2020) First version.

"""


import numpy as np
import scipy.special as ss


def PSDMatern (s,L,nu,D=1):
    expfct = nu + D/2
    fnum = 2**D * np.pi**(D/2)*ss.gamma(expfct)*(2*nu)**nu
    fdem = ss.gamma(nu)*L**(2*nu)
    tterm = ((2*nu)/L**2 + 4*np.pi**2*s**2)**-(nu+D/2)
    #
    return fnum * tterm / fdem



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 19/11/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

			d = {'a':dt['a'], "b":dt['b']}
			df = pd.DataFrame(data=d)

			d1 = df['a']
			d2 = df['b']

			rs = [crosscorr(d1,d2, lag) for lag in range(-N,N)]

Remarks : 

History : (19/11/2020) First version.

"""


def Pandas_CCF (datax, datay, lag=0, wrap=False):
    """ Lag-N cross correlation. 
    Shifted data filled with NaNs 
    
    Parameters
    ----------
    lag : int, default 0
    datax, datay : pandas.Series objects of equal length
    Returns
    ----------
    crosscorr : float
    """
    if wrap:
        shiftedy = datay.shift(lag)
        shiftedy.iloc[:lag] = datay.iloc[-lag:].values
        return datax.corr(shiftedy)
    else: 
        return datax.corr(datay.shift(lag))



""" 

Context : SRP
Module  : MCMC (Monte Carlo Markov Chain)
Version : 1.1.0
Author  : Stefano Covino
Date    : 21/08/2020
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (10/08/2020) First version.
        : (21/08/2020) ptmcee_run added.
"""



__all__ = ['emcee_run', 'ptemcee_run', 'quartile']




""" Utility functions and classes for SRP

Context : SRP
Module  : Statistics.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 27/03/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input data are lists of x and weights

History : (27/08/2012) First version.
        : (27/03/2013) Bug corrections.

"""



import numpy
from .WeightedMeanFrame import WeightedMeanFrame





def AverSigmaClippFrameFast (x, wx=None, downsig=None, upsig=None):
    nx = []
    nwx = []
    # generate numpy arrays
    for i in range(len(x)):
        nx.append(numpy.array(x[i]))
        if wx != None:
            nwx.append(numpy.array(wx[i]))
        else:
            nwx.append(numpy.ones(nx[0].shape))
    #
    res = WeightedMeanFrame(nx,nwx)
    wa = res[0]
    ws = res[1]
    we = res[2]
    wexp = res[3]
    #
    if (downsig == None and upsig == None) or (len(nx) == 1):
        # no condition, nothing to do
        return wa,ws,we,wexp
    # generate arrays of conditions
    ncond = []
    for i in range(len(nx)):
        if downsig != None and upsig == None:
            # high-pass filter
            ncond.append(numpy.where(nx[i] >= (wa-ws*downsig), 1., 0.))
        elif downsig == None and upsig != None:
            # low-pass filter
            ncond.append(numpy.where(nx[i] <= (wa+ws*upsig), 1., 0.))
        else:
            # sigma-clipping
            ncond.append(numpy.where((nx[i] >= wa-ws*downsig) & (nx[i] <= wa+ws*upsig), 1., 0.))
    # generate conditionates arrays
    nxt = []
    nwxt = []
    for i in range(len(x)):
        nxt.append(nx[i] * ncond[i])
        nwxt.append(ncond[i] * nwx[i])
    #
    res = WeightedMeanFrame(nxt,nwxt)
    wa = res[0]
    ws = res[1]
    we = res[2]
    wexp = res[3]
    #
    return wa,ws,we,wexp
    # era wa, ncond
    #




""" 

Context : SRP
Module  : GP
Version : 1.0.0
Author  : Stefano Covino
Date    : 30/01/2020
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (30/01/2020) First version.
"""



__all__ = ['AEKernel', 'CosineKernel', 'ExpSineKernel',
            'MaternKernel', 'PSDMatern', 'PSDSE', 'RQKernel',
            'SEKernel']




""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 10/08/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : quartile (samples)
        : sampples: outpout of a MCMC chain
        : samples = sampler.chain[:, nburnt:, :].reshape((-1,ndim))

History : (10/08/2020) First version.

"""

import numpy as np


def quartile (samples, perc=[15.9, 50.0, 84.1, 99.]):
    #
    res = np.percentile(samples, perc, axis=0)
    # 
    return res


""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 21/08/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : ptmcee MCMC
        : parameter:
           ndim -  number of dimension of the problem
           nwalkers - number of walkers
           ntemps - number of temperatures
           starts - starting positions of the chain
           sigmas - sigmas for the starting Gaussian ball (e.g. 0.1*starts)
           nburnt - ingterations to reach stability
           niters - total iterations
           lnprob - log of probability to sample
           args - [parameters for lnprob]


        : Example of code
          def lnlike(theta, t, Io, Do, Ro):
            alpha = np.exp(theta[0])
            beta = np.exp(theta[1])
            deltat = int(round(theta[2]))
            fact = np.exp(theta[3])
            taub = np.exp(theta[4])
            mu = np.exp(theta[5])
            t0b = theta[6]
            factr = np.exp(theta[7])
            t0m = theta[8]    
            taum = np.exp(theta[9])
            ti = t - deltat
            s,i,r,d,n = sird(alpha,beta,mu,deltat,taub,t0b,taum,t0m,0,ti[-1],1)
            ichi = -0.5 * np.sum(np.log(2 * np.pi * (fact*np.sqrt(Io)) ** 2) + (i[ti[0]-1:ti[-1]] - Io*fact) ** 2 / (fact*np.sqrt(Io)) ** 2)
            rchi = -0.5 * np.sum(np.log(2 * np.pi * (factr*fact*np.sqrt(Ro)) ** 2) + (r[ti[0]-1:ti[-1]] - Ro*factr*fact) ** 2 / (factr*fact*np.sqrt(Ro)) ** 2)
            dm = np.round(d)
            dchi = np.sum(poisson.logpmf(dm[ti[0]-1:ti[-1]],Do))
            return ichi + dchi + rchi


            def lnprior(theta):
                #
                prior = 0.
                #
                alpha = np.exp(theta[0])
                beta = np.exp(theta[1])
                deltat = theta[2]
                fact = np.exp(theta[3])
                taub = np.exp(theta[4])
                mu = np.exp(theta[5])
                t0b = theta[6]
                factr = np.exp(theta[7])
                t0m = theta[8]
                taum = np.exp(theta[9]) 
                #
                if alpha < 0.0001 or alpha > 1.:
                    return -np.inf
                #
                #lgb = np.log(gp((0.406/N_0,0.1/N_0),beta))
                #prior += lgb    
                #
                if fact < 1:
                    return -np.inf
                #
                # R0
                if beta*N_0/alpha <= 0.005 or beta*N_0/alpha > 30:
                    return -np.inf
                #
                #lgg = np.log(gp((26,0.1),deltat))
                if deltat < 1 or deltat > 50:
                    return -np.inf
                #
                lgtaub = np.log(gp((26,1),taub))
                prior += lgtaub
                #
                #lgm = np.log(gp((0.014,0.001),mu/fact))
                #prior += lgm
                #
                cfr = mu/(alpha+mu)
                lgc = np.log(gp((0.0114,0.0001),cfr))
                prior += lgc
                #if cfr < 0.001 or cfr > 0.01:
                #    return -np.inf
                lgtb = np.log(gp((60,1),t0b))
                prior += lgtb
                #
                if factr < 1:
                    return -np.inf
                #
                lgtaum = np.log(gp((19,1),taum))
                prior += lgtaum
                #
                lgtm = np.log(gp((90,0.1),t0m))
                prior += lgtm
                #
                return prior
                  

                def lnprob(theta, t, i, d, r):
                    lp = lnprior(theta)
                    if not np.isfinite(lp):
                        return -np.inf
                    return lp + lnlike(theta, t, i, d, r)

                #sampler = ptemcee.Sampler(nwalkers, ndim, logl=lnlike, logp=lnprob, ntemps=ntemps, logpargs=(nep['DOrd'][startd:endd], nep['Infected'][startd:endd], nep['Dead'][startd:endd], nep['Healed'][startd:endd]), loglargs=(nep['DOrd'][startd:endd], nep['Infected'][startd:endd], nep['Infected'][startd:endd], nep['Healed'][startd:endd]))


History : (21/08/2020) First version.

"""

import emcee
import ptmcee
import numpy as np


def ptmcee_run (ndim,nwalkers,ntemps,starts,sigmas,nburnt,niters,lnlike,lnprob,llargs,lpargs):

    pos = np.array(starts) + np.array(sigmas)*np.random.randn(ntemps,nwalkers,ndim)

    sampler = ptmcee.Sampler(nwalkers, ndim, logl=lnlike, logp=lnprob, ntemps=ntemps, loglargs=llargs, logpargs=lpargs)

    sampler.run_mcmc(pos, niters)

    samples = sampler.chain[:, :, nburnt:, :].reshape((-1,ndim))
    
    logevidence = sampler.log_evidence_estimate()

    meanaccfract = np.mean(sampler.acceptance_fraction)

    return {"sampler": sampler, "meanacceptancefraction": meanaccfract, "samples": samples, "logevidence":logevidence}





""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.2
Author  : Stefano Covino
Date    : 11/12/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (24/00/2010) First version.
        : (16/05/2017) Minor update.
		: (11/12/2020) Minor corretion for using astLib.
"""


import math
import numpy
import scipy.optimize as so

import astLib.astCoords as aLaC

from SRPFITS.Frames.Pixel2WCS import Pixel2WCS


def SkyDistSum (pars, oldvars, newvars, head):
    tot = 0.0
    head.UpgradeWCSHeader(pars)
    newc = Pixel2WCS(head.FitsFrame.Header,oldvars,'astlib')
    for i,l in zip(newc,newvars):
        tot = tot + (aLaC.calcAngSepDeg(i[0],i[1],l[0],l[1]))*3600.0
    return tot


def SkyDistanceMinimization (guessparset, oldcoord, refcoord, head):
    rc = numpy.array(refcoord)
    oc = numpy.array(oldcoord)
    xopt = so.fmin(SkyDistSum, guessparset, args=(oc,rc,head), disp=0)
    return xopt, SkyDistSum(xopt,oc,rc,head)/len(oc)




""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 04/12/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : inputs are a list of parameters and a list of choices for 
            the fit: e.g. [(1,0.2),(1,-0.34),(0,0.01),(-1,0.2)]

History : (04/12/2012) First version.

"""


def GenFitPars (pars, args):
    pari = []
    if len(pars) != len(args):
        raise Exception("Input vectors must be of the same length.")
    for ii in range(len(pars)):
        if args[ii][0] > 0:
            pari.append(pars[ii])
        elif args[ii][0] == 0:
            pari.append(args[ii][1])
        else:
            if not (1 <= abs(args[ii][0]) <= len(pars)):
                raise Exception("Parameter to couple not existent.")
            pari.append(pars[abs(args[ii][0])-1])
    return pari





""" Utility functions and classes for SRP

Context : SRP
Module  : Statsistics
Version : 1.0.0
Author  : Stefano Covino
Date    : 04/04/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : inputs are a 1D vectors to be cross-correlated. Optionally you can
            give a vector of x-axis units. It returns the cross-correlation 
            value.

History : (04/04/2013) First version.

"""

import numpy


def XCorr_1D (data, refdata, xdata=None):
    if data.ndim == 1 and refdata.ndim == 1:
        ycorr = numpy.correlate(data, refdata, mode="full")
        xcorr = numpy.arange(ycorr.size)
        lags = xcorr - (data.size-1)
        if xdata == None:
            distPerLag = 1.
        elif xdata.ndim == 1:
            distPerLag = (xdata[-1] - xdata[0])/float(xdata.size)
        else:
            return None
        #
        offsets = -lags*distPerLag
        #
        mx = ycorr.argmax()
        ox = offsets[mx]
        return ox
    else:
        return None



""" 

Context : SRP
Module  : FT (Fourier Transform)
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/02/2020
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (03/02/2020) First version.
"""



__all__ = ['ACF', 'fftransform', 'FourierPeriodogram']




""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 30/04/2022
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : Akaike (loglikelihood,npars)
            This function reports the Akaike information criterion knowking the best-fit Log Likelihoood,
            and the number of fit parameters.

Remarks :

History : (30/04/2022) First version.

"""




def Akaike(LLike,npars):
    return -2*LLike+2*npars


    
    



from setuptools import setup, find_packages
from os import path
import glob
import SRPSTATS
import os

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


# Look for script files
lscr = glob.glob(os.path.join('Scripts', 'SRP*'))
lscrex = []
for i in lscr:
    if os.path.splitext(i)[1] == '':
        lscrex.append(i)


setup(
    name='SRPAstro.STATS', 
    version=SRPSTATS.__version__, 
    description='Statistical tools under SRP', 
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://pypi.python.org/pypi/SRPAstro.STATS',
    author='Stefano Covino',
    author_email='stefano.covino@inaf.it',
    classifiers=[
                   'Development Status :: 5 - Production/Stable',
                   'Environment :: Console',
                   'Intended Audience :: Science/Research',
                   'Topic :: Scientific/Engineering :: Astronomy',
                   'License :: OSI Approved :: MIT License',
                   'Operating System :: MacOS :: MacOS X',
                   'Operating System :: Unix',
                   'Operating System :: POSIX',
                   'Programming Language :: Python :: 3',
                   ],
    keywords='astronomy data analysis',
    packages=find_packages(),
    python_requires='>=3',
    scripts=lscrex,
    install_requires=['SRPAstro','emcee>=3','ptemcee','scipy','numpy','pandas'],
    ) 




""" Utility functions and classes for SRP

Context : SRP
Module  : GP
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/01/2020
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : From C. E. Rasmussen & C. K. I. Williams, Gaussian Processes
            for Machine Learning, the MIT Press, 2006

History : (29/01/2020) First version.

"""


import numpy as np
import scipy.special as ss


def MaternKernel (A,r,L,nu):
    fi = (2**(1-nu))/ss.gamma(nu)
    sem = np.sqrt(2*nu)*r/L
    se = sem**nu
    te = ss.kv(nu,sem)
    return A*fi*se*te



""" Utility functions and classes for SRP

Context : SRP
Module  : Stats
Version : 1.0.0
Author  : Stefano Covino
Date    : 31/07/2020
E-mail  : stefano.covino@inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : SSE(model, data)


History : (31/07/2020) First version.
"""

#from scipy import stats


def SSE (model, data):
    res = ((data - model)/model)**2
    if len(res.shape) == 2:
        return res.sum(axis=1)
    else:
        return res.sum()


""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 30/04/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : ChiSqIncrement(dof,prob=90,acc=0.001)
            dof is the number of degrees of freedom
            prob is the probability for having a higher chisquare than 100-prob%
            acc is the accuracy of the chisquare increment computation
          The function return the chisquare increment.
        

Remarks : input is a list of (x,sigmax)

History : (30/04/2011) First version.
        : (24/07/2015) scipy porting.
"""

from scipy import stats


def ChiSqIncrement(dof, prob=90.0, acc=0.001):
    if dof <= 0:
        return 0.0
    alf = 100.0 - prob
    maxchi = 0.0
    chii = 0.1
    upi = True
    while not alf-acc/2. <= 100.0*(1.-stats.chi2.cdf(maxchi,dof)) <= alf+acc/2.:
        oldi = upi      # save the old increment direction
        if 100.0*(1.-stats.chi2.cdf(maxchi,dof)) > alf:
            maxchi = maxchi + chii
            upi = True          # up increment
        else:
            maxchi = maxchi - chii
            upi = False         # down increment
            if maxchi < 0:
                maxchi = maxchi + chii  # back to original value
                chii = chii / 2.0       # if increment gets negative better refined serach
        if oldi != upi:
            chii = chii/2.      # if oscillates half inrement
    return maxchi
    
    


""" 

Context : SRP
Module  : Statistics
Author  : Stefano Covino
Date    : 30/04/2022
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (28/10/2010) First version.
        : (30/04/2011) ChiSqIncrement.
        : (19/02/2012) ScoreatPercentile.
        : (27/08/2012) Faster sigma clipping for frames.
        : (04/03/2013) MAD.
        : (04/04/2013) XCorr_1D.
        : (07/06/2013) FTest.
        : (22/12/2014) RebinData.
        : (08/11/2015) StDev.
        : (02/11/2016) FourierPeriodogram.
        : (04/10/2019) Bayes factor.
        : (29/01/2020) GP and FT library.
        : (08/04/2020) Stand-alone package and MCMC library.
        : (31/07/2020) chi2 and SSE.
        : (13/08/2020) p-value and Bayes p-value
        : (21/08/2020) ptmcee_run
        : (19/11/2020) pandas based CCF.
        : (11/12/2020) Minor correction.
        : (21/01/2021) Minor correction in emcee_run and ptemcee_run.
        : (12/05/2021) Correction for pypi effective download.
        : (18/05/2021) Improved ptemcee_run.
        : (19/05/2021) Improved emcee_run
        : (03/06/2021) Bug corrections in porting to SRPSTATS.
        : (12/11/2021) Error condition intercepted in MCMC.
        : (18/11/2021) Minor correction to Gaussian sample generation.
        : (17/01/2022) Improvement to RebinData and BIC.
        : (30/04/2022) Akiake information criterion.
"""


__version__ = '1.4.0'


__all__ = ['Akiake', 'AsymmetricGaussianDistrib', 'AverIterSigmaClipp', 
		'AverSigmaClippFrame',  'AverSigmaClippFrameFast', 
		'BayesFactor', 'Bayespvalue', 'BIC', 'chi2', 'Pandas_CCF', 
		'ChSqIncrement', 'CoordDistanceMinimization', 'FT', 'FTest', 
		'GenFitPars', 'GenGaussSet', 'GP', 'MAD', 'MCMC', 'pvalue', 
		'RebinData', 'ScoreatPercentile', 'SSE', 'StDev', 'XCorr_1D', 
		'WeightedMean', 'WeightedMeanFrame']




""" Utility functions and classes for SRP

Context : SRP
Module  : Stats.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 07/06/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Ref: http://it.wikipedia.org/wiki/Test_F

History : (07/06/2013) First version.

"""


def FTest (oldchi2, olddof, newchi2, newdof):
    return ((oldchi2-newchi2)/(olddof-newdof))/(newchi2/newdof)